aboutsummaryrefslogtreecommitdiffstats
path: root/target/linux/brcm2708/patches-4.14/950-0272-net-lan78xx-Reduce-s-w-csum-check-on-VLANs.patch
blob: 859cca21360a1ae741f5144773b7b79c46dfcb63 (plain)
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
From 126c6760f56d2d7a82c605a54c402a886ae9b5bf Mon Sep 17 00:00:00 2001
From: Dave Stevenson <dave.stevenson@raspberrypi.org>
Date: Tue, 10 Apr 2018 13:08:36 +0100
Subject: [PATCH 272/454] net: lan78xx: Reduce s/w csum check on VLANs

With HW_VLAN_CTAG_RX enabled we don't observe the checksum
issue, so amend the workaround to only drop back to s/w
checksums if VLAN offload is disabled.

See #2458.

Signed-off-by: Dave Stevenson <dave.stevenson@raspberrypi.org>
---
 drivers/net/usb/lan78xx.c | 7 ++++---
 1 file changed, 4 insertions(+), 3 deletions(-)

--- a/drivers/net/usb/lan78xx.c
+++ b/drivers/net/usb/lan78xx.c
@@ -2972,12 +2972,13 @@ static void lan78xx_rx_csum_offload(stru
 				    struct sk_buff *skb,
 				    u32 rx_cmd_a, u32 rx_cmd_b)
 {
 	/* Checksum offload appears to be flawed if used with VLANs.
 	 * Elect for sw checksum check instead.
 	 */
 	if (!(dev->net->features & NETIF_F_RXCSUM) ||
 	    unlikely(rx_cmd_a & RX_CMD_A_ICSM_) ||
-	    (rx_cmd_a & RX_CMD_A_FVTG_)) {
+	    ((rx_cmd_a & RX_CMD_A_FVTG_) &&
+	     !(dev->net->features & NETIF_F_HW_VLAN_CTAG_RX))) {
 		skb->ip_summed = CHECKSUM_NONE;
 	} else {
 		skb->csum = ntohs((u16)(rx_cmd_b >> RX_CMD_B_CSUM_SHIFT_));
@@ -2972,12 +2972,13 @@ static void lan78xx_rx_csum_offload(stru
 				    u32 rx_cmd_a, u32 rx_cmd_b)
 {
-	/* Checksum offload appears to be flawed if used with VLANs.
-	 * Elect for sw checksum check instead.
+	/* HW Checksum offload appears to be flawed if used when not stripping
+	 * VLAN headers.
 	 */
 	if (!(dev->net->features & NETIF_F_RXCSUM) ||